package name.panitz.parser;
import java.util.*;
import static name.panitz.parser.Parser.*;
public class Eval{
  int eval(Prog prog){
    var funs = new HashMap<String,Fun>();
    prog.funs().forEach(fun-> funs.put(fun.name(),fun));
    return eval(funs,new HashMap<>(),prog.main());
  }


  int eval(Map<String,Fun> funs,Map<String,Integer> vars ,AST expr){
    if (expr instanceof IntLiteral il) return il.n();
    if (expr instanceof Var v) return vars.get(v.n());
    if (expr instanceof OpExpr op)
     return op.op().op.apply(eval(funs,vars,op.l()),eval(funs,vars,op.r()));
    if (expr instanceof Assign as) {
      var r = eval(funs,vars,as.r());
      vars.put(as.name(),r);
      return r;
    }
    if (expr instanceof Sequence es) {
      var r = 0;
      for (var e:es.sts()){
         r = eval(funs,vars,e);
      }
      return r;
    }
    if (expr instanceof If ife) 
      return (eval(funs,vars,ife.cond())!=0)
          ?   eval(funs,vars,ife.then())
          :   eval(funs,vars,ife.otherwise());
    if (expr instanceof While we) {
      var r = 0;
      var res = 0;
      while ((r=eval(funs,vars,we.cond()))!=0){
        res=eval(funs,vars,we.body());
      }
      return res;
    }
    if (expr instanceof FunCall fc) {
      var v2 = new HashMap<String,Integer>();
      var fun = funs.get(fc.name());
      var args = fun.args().iterator();
      for (var p:fc.args())v2.put(args.next(),eval(funs,vars,p));
      return eval(funs,v2,fun.body());
    }

    throw new RuntimeException("unknown pattern class: "+expr.getClass());
  }
}